package cambiavalute.model;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import java.util.Map;
import java.util.OptionalDouble;

public class CambiaValute {
	private Map<String,TassiDiCambio> elencoValute;
	private String nomeAgenzia;
	
	public CambiaValute(String nomeAgenzia, Map<String,TassiDiCambio> elencoValute) {
		this.nomeAgenzia = nomeAgenzia;
		this.elencoValute = elencoValute;
	}

	public OptionalDouble vendita(String siglaValutaEstera, double importo){
		TassiDiCambio tassiVigenti = elencoValute.get(siglaValutaEstera);
		if(tassiVigenti!= null)	return OptionalDouble.of(arrotonda(importo * tassiVigenti.getPrezzoVendita()));
		else return OptionalDouble.empty();
	}
	
	public OptionalDouble acquisto(String siglaValutaEstera, double importo){
		TassiDiCambio tassiVigenti = elencoValute.get(siglaValutaEstera);
		if(tassiVigenti!= null)	return OptionalDouble.of(arrotonda(importo / tassiVigenti.getPrezzoAcquisto()));
		else return OptionalDouble.empty();
	}

	public static double arrotonda(double importo){
		return Math.rint(importo*100)/100;
	}
	
	public static String formatta(double importo){
		//NumberFormat formatter = NumberFormat.getNumberInstance(Locale.UK);
		NumberFormat formatter = NumberFormat.getNumberInstance(Locale.ITALY);
		formatter.setMaximumFractionDigits(2);
		formatter.setMinimumFractionDigits(2);
		return formatter.format(importo);
	}
	
	public static double convertiInDouble(String importo) throws ParseException{
		int decPointPos = importo.indexOf(".");
		if (decPointPos>=0) throw new ParseException("Punto decimale non ammesso",decPointPos);
		NumberFormat formatter = NumberFormat.getNumberInstance(Locale.ITALY);
		return formatter.parse(importo).doubleValue();
	}

	public String getNomeAgenzia() {
		return nomeAgenzia;
	}

	public Map<String,TassiDiCambio> getElencoValute() {
		return elencoValute;
	}
	
	@Override
	public String toString(){
		return nomeAgenzia;
	}
}
