package cambiavalute.persistence;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import cambiavalute.model.*;

public class MyCambiReader implements CambiReader {

	private Map<String, TassiDiCambio> elencoCambi;

	public MyCambiReader() {
	}

	public Map<String, TassiDiCambio> leggiTabellaCambiApplicati(Reader reader)
			throws BadFileFormatException {
		if (reader == null)
			throw new IllegalArgumentException("reader");

		elencoCambi = new HashMap<>();
		try {
			BufferedReader bufferedReader = new BufferedReader(reader);
			NumberFormat formatter = NumberFormat
					.getNumberInstance(Locale.ITALY);
			String line;
			while ((line = bufferedReader.readLine()) != null) {
				StringTokenizer tokenizer = new StringTokenizer(line, " \t");
				String siglaValuta = tokenizer.nextToken().trim();
				String weSell = tokenizer.nextToken().trim();
				String weBuy = tokenizer.nextToken().trim();
				// verifiche consistenza formato
				if (siglaValuta.length() != 3)
					throw new BadFileFormatException("sigla valuta illegale");
				// lettura valori con Locale italiano
				double prezzoVendita = formatter.parse(weSell).doubleValue();
				double prezzoAcquisto = formatter.parse(weBuy).doubleValue();
				TassiDiCambio cambiPerValuta = new TassiDiCambio(siglaValuta,
						prezzoAcquisto, prezzoVendita);
				elencoCambi.put(siglaValuta, cambiPerValuta);
			}
		} catch (IOException | ParseException | NoSuchElementException e) {
			throw new BadFileFormatException("Errore lettura cambi");
		}
		return elencoCambi;

	}

	public static void main(String[] args) throws BadFileFormatException, IOException {
		CambiReader reader = new MyCambiReader();
		try (FileReader fileReader = new FileReader("EDBank.txt")) {
			Map<String, TassiDiCambio> mappa = reader.leggiTabellaCambiApplicati(fileReader);
			for (String sigla : mappa.keySet())
				System.out.println(mappa.get(sigla));
		}
	}

}
